#!/usr/local/bin/php -q
<?PHP
  $address = '0.0.0.0';
  $port = 4573;

  $pid_file = '/tmp/phpfastagi.pid';

  require_once(dirname(__FILE__) . DIRECTORY_SEPARATOR . 'phpagi.php');

  error_reporting(E_ALL);
  set_time_limit(0);

  if(file_exists($pid_file))
  {
    $fp = fopen($pid_file, 'r');
    $pid = fgets($fp, 1024);
    fclose($fp);
    if(posix_kill($pid, 0)) 					exit("Server already running with PID: $pid");
    if(!unlink($pid_file)) 					exit("Cannot unlink PID file $pid_file");
  }

  pcntl_signal(SIGTERM, 'phpfastagi_shutdown');
  function phpfastagi_shutdown($sig_no=SIGTERM)
  {
    global $sock, $pid, $pid_file;
    if($sock) socket_close($sock);
    if(posix_getpid() == $pid)
      @unlink($pid_file);
    echo "daemon stopped\n";
    exit;
  }

  if(($sock = socket_create(AF_INET, SOCK_STREAM, 0)) < 0)	exit("socket_create() failed: " . socket_strerror($sock));
  if(($ret  = socket_bind($sock, $address, $port)) < 0)		exit("socket_bind() failed: "   . socket_strerror($ret));
  if(($ret  = socket_listen($sock, 0)) < 0)			exit("socket_listen() failed: " . socket_strerror($ret));

  socket_set_nonblock($sock);

  if(!($fp = fopen($pid_file, 'w')))				exit("Unable to open PID file $pid_file for writing...");
  fputs($fp, posix_getpid());
  fclose($fp);

  echo "Waiting for connections.....\n";

  $cstat = $remote_ip = $remote_port = NULL;
  do
  {
    do
    {
      usleep(1000);
      pcntl_waitpid(-1, $cstat, WNOHANG);
    }while(!is_resource($connection = @socket_accept($sock)));

    $pid = pcntl_fork();
    if($pid === 0)
    {
      @socket_getpeername($connection, $remote_ip, $remote_port);

      $fastagi = new AGI(null, array(), $connection);

      $fastagi->request['CLIENT_ADDRESS'] = NULL;
      socket_getpeername($connection, $fastagi->request['CLIENT_ADDRESS']);

      if(!isset($fastagi->config['fastagi']['basedir']))
        $fastagi->config['fastagi']['basedir'] = dirname(__FILE__);

      // perform some security checks

      $script = $fastagi->config['fastagi']['basedir'] . DIRECTORY_SEPARATOR . $fastagi->request['agi_network_script'];

      // in the same directory (or subdirectory)
      $mydir = dirname($fastagi->config['fastagi']['basedir']) . DIRECTORY_SEPARATOR;
      $dir = dirname($script) . DIRECTORY_SEPARATOR;
      if(substr($dir, 0, strlen($mydir)) != $mydir)
      {
        $fastagi->conlog("$script is not allowed to execute.");
        socket_close($connection);
        exit;
      }

      // make sure it exists
      if(!file_exists($script))
      {
        $fastagi->conlog("$script does not exist.");
        socket_close($connection);
        exit;
      }

      // drop privileges
      if(isset($fastagi->config['fastagi']['setuid']) && $fastagi->config['fastagi']['setuid'])
      {
        $owner = fileowner($script);
        $group = filegroup($script);
        if(!posix_setgid($group) || !posix_setegid($group) || !posix_setuid($owner) || !posix_seteuid($owner))
        {
          $fastagi->conlog("failed to lower privileges.");
          socket_close($connection);
          exit;
        }
      }

      // make sure script is still readable
      if(!is_readable($script))
      {
        $fastagi->conlog("$script is not readable.");
        socket_close($connection);
        exit;
      }

      require_once($script);

      socket_close($connection);
      exit;
    }
  } while($pid != -1)
  if($pid == -1)
    exit('ERROR: unable to pcntl_fork()');
?>
